#!/bin/sh

# standard ls command is left untouched
alias l='ls -laF --color=tty'      # colored full ls
alias lm='l --color=always |more'  # colored full ls + more
alias ll='l |less'
alias m=". /usr/share/mc/bin/mc-wrapper.sh"
alias v=/etc/alternatives/vim
#
alias ..='cd ..'
alias ...='cd ./../..'
alias ....='cd ./../../..'
alias .....='cd ./../../../..'
alias cd..='cd ..'
alias md='mkdir -p'
alias rd=rmdir
#
alias mm='mount'
alias um='umount'
#
alias psa='ps -lA'
alias psag='ps -lA |grep '
#
alias tk='tkate.sh'		# open Kate from terminal
alias gd='gdb -x /home/linux/setup/gdb_startup.gdb'
#
alias did='dpkg -i *.deb'
alias did..='dpkg -i ../*.deb'
alias drb='debian/rules binary'
alias drbi='drb && did..'
alias drc='debian/rules clean'
alias drcb='drc && drb'
alias drcbi='drc && drbi'
alias pbu='pbuilder update'
alias pdr='patch --dry-run -p '
alias ptc='patch -p '
#
alias ntt='ninja -t targets'

#--------------------------------------------------------------------------
# fuzzy file finding
#
fcd()
{
	f=`find ${1:-"."} -type d | fzf`
	if [ -d "$f" ]; then
		cd "$f"
	fi
}

ffd()
{
	find . -type d | fzf 
}

fff()
{
	find . -type f | fzf 
}

#--------------------------------------------------------------------------
# fuzzy history
#
hf()
{
	#history | fzf | awk '{$1= ""; print $0}' | xargs -I{} bash -ic '{}' 
	history | fzf | awk '{$1= ""; print $0}' | sh
}

hfs()
{
	history | fzf | awk '{$1= ""; print $0}'
}

#--------------------------------------------------------------------------
# git commands
#

USE_GIT_WORKTREES="y"
WORKTREES_DIR="/home/tde_src/1_git/worktrees"

# branch list (locane and remote)
gbr()
{
	git branch "$@"
}

gbrr()
{
	git branch --remote
}

# worktree list
gwl()
{
	git worktree list
}

# worktree prune
gwp()
{
	git worktree prune
}

# git status and logs
gst()
{
	git status
}

gld()
{
	git ld
}

gll()
{
	if [[ -n "$1" ]]; then
		git ll -n "$@"
	else
		git ll
	fi
}

glp()
{
	git lp
}

# git diff of uncached/cached files
gdu()
{
	git diff
}

gdc()
{
	git dfc
}

# git add all changes
gaa()
{
	git add -A
}

# git checkout
gck()
{
	git checkout "$@"
}

# checkout branch/wortree
#  $1 - the branch to checkout
#  $2 - the name of a new branch/worktree. If not an empty string, a copy of the branch in a new worktree if created
git_ck_branch()
{
	local BRANCH="$1"
	if [ "${USE_GIT_WORKTREES}" = "y" ]; then
		local BRANCH_WORKTREE=`git worktree list | grep "\[${BRANCH}\]" | perl -p -e "s|^(.*?)( +)[0-9a-fA-F]{7,} .*|\1|"`
		if [ -n "${BRANCH_WORKTREE}" -a -z "$2" ]; then
			# Worktree already exists
			if [ -d "${BRANCH_WORKTREE}" ]; then
				cd "${BRANCH_WORKTREE}" &>/dev/null
			else
				echo "Unable to find worktree folder ($BRANCH_WORKTREE)"
			fi
		else
			# Worktree does not exist or a new branch is requested
			local REPOBASE=$(basename `git rev-parse --show-toplevel`)
			local WORK_PATH=""
			if [[ -n "$2" ]]; then
				WORK_PATH="${WORKTREES_DIR}/others/${REPOBASE}/$2/BRANCH"
				if [ ! -d "${WORK_PATH}" ]; then
					git worktree add -b "$2" --no-track  "${WORK_PATH}" "origin/${BRANCH}"
					cd "${WORK_PATH}" &>/dev/null
				else
					echo "Worktree folder already exists (${WORK_PATH})"
				fi
			else
				WORK_PATH="${WORKTREES_DIR}/others/${REPOBASE}/${BRANCH}/BRANCH"
				if [ ! -d "${WORK_PATH}" ]; then
					git worktree add "${WORK_PATH}" "origin/${BRANCH}"
					cd "${WORK_PATH}" &>/dev/null
					git checkout ${BRANCH}
					git branch --set-upstream-to=origin/${BRANCH}
				else
					echo "Worktree folder already exists (${WORK_PATH})"
				fi
			fi
		fi
	else
		git checkout "${BRANCH}"
	fi
}

# checkout local branch/wortree
gcb()
{
  if [[ -z `git branch |& grep "^fatal:"` ]]; then
    local BRANCH=`git branch | fzf | sed "s|^[*+ ] ||"`
		if [[ -n "${BRANCH}" ]]; then
		  git_ck_branch "$BRANCH" "$1"
		fi
	else
		echo "Not on a git repository"
	fi
}

# checkout remote branch/worktree
gcrb()
{
  if [[ -z `git branch |& grep "^fatal:"` ]]; then
    local BRANCH=`git branch --remote | fzf | sed "s|^ *origin/||"`
		if [[ -n "${BRANCH}" ]]; then
		  git_ck_branch "$BRANCH" "$1"
		fi
	else
		echo "Not on a git repository"
	fi
}

# remove local branch
grb()
{
  if [[ -z `git branch |& grep "^fatal:"` ]]; then
    local BRANCH_STRING=`git branch | fzf`
    local BRANCH=`echo "${BRANCH_STRING}" | sed "s|^[*+ ] ||"`
		if [[ "${BRANCH_STRING}" =~ \*.* ]]; then  # this check if the branch string starts with *
			echo "Can't delete active branch (${BRANCH})"
		else
			git branch -D "${BRANCH}" && git branch
		fi
	else
		echo "Not on a git repository"
	fi
}

# remove worktree
grw()
{
	if [ "${USE_GIT_WORKTREES}" = "y" ]; then
		if [[ -z `git branch |& grep "^fatal:"` ]]; then
			local WORKTREE_PATH=`git worktree list | grep -v "\(bare\)" | fzf| perl -p -e "s|^(.*?)( +)[0-9a-fA-F]{7,} .*|\1|"`
			if [[ -n "${WORKTREE_PATH}" ]]; then
				if [[ "`pwd`" == "${WORKTREE_PATH}" ]]; then
			    echo "Can't delete active worktree"
				else
					if [ -d "${WORKTREE_PATH}" ]; then
					  git worktree remove "${WORKTREE_PATH}" &&
  					rmdir -p `dirname "${WORKTREE_PATH}"` &>/dev/null
 					else
				    echo "Unable to find worktree folder (${WORKTREE_PATH})"
					fi
				fi
			fi
		else
			echo "Not on a git repository"
		fi
	else
		echo "Command not available is not using git worktrees"
	fi
}

# checkout master and rebase
gcm()
{
  if [[ -z `git branch |& grep "^fatal:"` ]]; then
	  git_ck_branch "master" && git rebase -S
	else
		echo "Not on a git repository"
	fi
}

# checkout r14.1.x and rebase
gcx()
{
  if [[ -z `git branch |& grep "^fatal:"` ]]; then
		git_ck_branch "r14.1.x" && git rebase -S
	else
		echo "Not on a git repository"
	fi
}

# checkout fetch and rebase
gfar()
{
	if [[ -z `git branch |& grep "^fatal:"` ]]; then
		git fetch --all --prune
		echo "-----------------------"
		echo "Updating r14.1.x branch"
		gcx
		echo "-----------------------"
		echo "Updating master branch"
		gcm
	else
		echo "Not on a git repository"
	fi
}

# backport a commit
gbpc()
{
	if [ "$1" = "" ]; then
		echo "Please provide the required commit hash."
	elif [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
    local BRANCH=`git branch | grep "^*" | sed "s|^[*+ ] ||"`
		if [ "${BRANCH}" != "master" ]; then
		  echo "Not on master branch"
		else
		  git_ck_branch "r14.1.x"
			git cherry-pick -x -S $1
			if [ $? -eq 0 ]; then
				git push origin r14.1.x
				git_ck_branch "master"
			else
				echo "--- ERROR: cherry pick failed ---"
			fi
		fi
	fi
}

# git pom
gpom()
{
	if [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
    local BRANCH=`git branch | grep "^*" | sed "s|^* ||"`
		if [ "${BRANCH}" != "master" ]; then
		  echo "Not on master branch"
		else
		  git push origin master 
		fi
	fi
}

# git pox
gpox()
{
	if [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
    local BRANCH=`git branch | grep "^*" | sed "s|^* ||"`
		if [ "${BRANCH}" != "r14.1.x" ]; then
		  echo "Not on r14.1.x branch"
		else
		  git push origin r14.1.x
		fi
	fi
}

# git pfo
gpfo()
{
	if [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
    local BRANCH=`git branch | grep "^*" | sed "s|^* ||"`
		if [ "${BRANCH}" != "master" ]; then
		  echo "Not on master branch"
		else
		  git push -f origin HEAD
		fi
	fi
}

# git poh
gpoh()
{
	if [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
    local BRANCH=`git branch | grep "^*" | sed "s|^* ||"`
		if [ "${BRANCH}" != "master" ]; then
		  echo "Not on master branch"
		else
		  git push origin HEAD
		fi
	fi
}

# git reset hard to parent commit
grhp()
{
	if [[ -n `git branch |& grep "^fatal:"` ]]; then
		echo "Not on a git repository"
	else
		git reset --hard @^
	fi
}

#--------------------------------------------------------------------------
# go to tde folders
#
tde_go()
{
	local BRANCH="$1"
	local TARGET_DIR=""
	[ "$BRANCH" = "x" ] && BRANCH="r14.1.x"
	if [ -n "${BRANCH}" ]; then
	  TARGET_DIR="$2/${BRANCH}/$3"
	else
	  TARGET_DIR="$2/$3"
	fi
	if [ -d "${TARGET_DIR}" ]; then
  	cd $(find "${TARGET_DIR}" -type d | fzf)
  else
		echo "Invalid folder/branch specified (target: ${TARGET_DIR})"
	fi
}

tg()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "tde"
}

te()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "edeps"
}

tb()
{
	tde_go "$1" "/home/tde_src/2_build/build"
}

td()
{
	tde_go "$1" "/home/tde_src/2_build/debs"
}

tm()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "tde/main"
}

tp()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "tde/packaging"
}

tpd()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "tde/packaging/debian/_base"
}

tpu()
{
	local BRANCH=${1:-"master"}
	tde_go "${BRANCH}" "${WORKTREES_DIR}" "tde/packaging/ubuntu/_base"
}

#--------------------------------------------------------------------------
# Build commands for TDE

BUILDSCRIPT_DIR="/home/tde_src/buildscripts"

# build module
bm()
{
	${BUILDSCRIPT_DIR}/build_module.sh "$@"
}

# build current folder
bcf()
{
	${BUILDSCRIPT_DIR}/build_module.sh -p . "$@"
}

# update TDE repositories
utr()
{
	${BUILDSCRIPT_DIR}/update_repositories.sh "$@"
}

# create TDE repo
ctr()
{
	${BUILDSCRIPT_DIR}/create_repo.sh "$@"
}
