local cmd = vim.cmd
local opt = vim.opt

--  Leader keys
vim.g.mapleader = ' '

--  GUI cursor
opt['guicursor'] = ''

-- Allows neovim to access the system clipboard
opt['clipboard'] = 'unnamedplus'

-- Don't show things like -- INSERT -- anymore
opt['showmode'] = false

-- File type and syntax highlighting
opt['filetype'] = 'off'
opt['syntax'] = 'on'
cmd('filetype plugin indent on')

--  Disable backup files and no swap files
opt['backup'] = false
opt['swapfile'] = false

-- Ask for confirmation if necessary
opt['confirm'] = true

-- Wildmenu settings
opt['wildmenu'] = true
opt['wildmode'] = 'full'

-- No timeout for key sequences
opt['timeout'] = false
opt['ttimeout'] = false
opt['whichwrap'] = vim.o['whichwrap'] .. ',<,>,h,l,[,]'

-- Backspace character moves back at this positions
opt['backspace'] = { 'indent' , 'eol' , 'start' }

-- Do not change working directly automatically
opt['autochdir'] = false

-- Disable modeline mode to prevent potential attacks
opt['modeline'] = false
opt['modelineexpr'] = false

-- Show line numbers in relative mode by default
opt['number'] = true
opt['relativenumber'] = true

-- Set font and window attributes
opt['encoding'] = 'utf-8'
opt['fileencoding'] = 'utf-8'

-- Indentation settings and special characters
opt['tabstop'] = 2
opt['shiftwidth'] = 2
opt['softtabstop'] = 2
opt['expandtab'] = false
opt['smartindent'] = true
opt['listchars'] = { tab = '▸ ' , eol = '¬' }

-- Set color options
-- opt['termguicolors'] = true
cmd('highlight NonText    guifg=#5FFFFF')
cmd('highlight SpecialKey guifg=#5FFFFF')

-- Search options
opt['hlsearch'] = true
opt['incsearch'] = true
opt['ignorecase'] = true
opt['smartcase'] = true

-- Folding
opt['foldmethod'] = 'syntax'
opt['foldenable'] = false
-- autocmd BufRead * normal zR

-- Windows splits
opt['splitbelow'] = true
opt['splitright'] = true

-- Persistent undo (excluding temporary files)
opt['undofile'] = true
cmd([[
  augroup AGUndoFile
    autocmd!
    autocmd BufWritePre /tmp/* setlocal noundofile
  augroup END
]])

-- Enter insert mode directly when entering a terminal buffer
cmd([[
  augroup AGTerminal
    autocmd!
    autocmd TermOpen term://* startinsert
    autocmd WinEnter * if &buftype == 'terminal' | startinsert | endif
  augroup END
]])
