local fn = vim.fn
local install_path = fn.stdpath('data')..'/site/pack/packer/start/packer.nvim'
if fn.empty(fn.glob(install_path)) > 0 then
  packer_bootstrap = fn.system({'git', 'clone', '--depth', '1', 'https://github.com/wbthomason/packer.nvim', install_path})
end

vim.cmd([[
  augroup packer_user_config
  autocmd!
  autocmd BufWritePost 02_plugins.lua source <afile> | PackerSync
  augroup end
]])

-- Use a protected call so we don't error out on first use
local status_ok, packer = pcall(require, "packer")
if not status_ok then
  vim.notify("Packer package not found!")
  return
end

-- Have packer use a popup window
--packer.init
--{
--  display =
--  {
--    open_fn = function()
--      return require("packer.util").float {}
--    end,
--  }
--}

-- FZF MRU history length
vim.cmd([[
  let g:fzf_mru_max=2000    " Max nummber of entries in fzf-mru
]])

return packer.startup(function(use)
  -- Packer can manage itself
  use 'wbthomason/packer.nvim'

  -- plugins with common functions
  use "nvim-lua/popup.nvim" -- An implementation of the Popup API from vim in Neovim
  use "nvim-lua/plenary.nvim" -- Useful lua functions used ny lots of plugins
  use 'nvim-tree/nvim-web-devicons'

  -- allow you to easily manage external editor tooling
  -- *** these plugins must come before nvim-lspconfig ***
  use "williamboman/mason.nvim"
  use "williamboman/mason-lspconfig.nvim"

  -- LSP plugins
  use 'neovim/nvim-lspconfig'
  use 'hrsh7th/cmp-nvim-lsp'

  -- Completion stuff
  use 'hrsh7th/nvim-cmp'
  use 'hrsh7th/cmp-buffer'
  use 'hrsh7th/cmp-path'
  use 'hrsh7th/cmp-cmdline'
  use 'hrsh7th/cmp-nvim-lua'
  use "saadparwaiz1/cmp_luasnip"

  -- snippets
  use {"L3MON4D3/LuaSnip" }
  use "micheleCTDE/snippets"

  -- Lightline status line
  use 'itchyny/lightline.vim'
  use 'josa42/nvim-lightline-lsp'

  -- FZF integration
  use { 'junegunn/fzf', run = 'fzf#install()' }
  use { 'ibhagwan/fzf-lua' }
  use 'pbogut/fzf-mru.vim'

  -- DAP integration
  use 'mfussenegger/nvim-dap'
  use "rcarriga/nvim-dap-ui"

	-- nvimtree
  use 'nvim-tree/nvim-tree.lua'

  -- Various plugins from Tim Pope
  use 'tpope/vim-abolish'
  use 'tpope/vim-repeat'
  use 'tpope/vim-surround'
  use 'tpope/vim-unimpaired'
  use { 'tpope/vim-fugitive',
    opt = true,
    cmd = { 'G', 'GStatus', 'Gclog', 'Gllog' }
  }


  -- Automatically set up your configuration after cloning packer.nvim
  -- Put this at the end after all plugins
  if packer_bootstrap then
    packer.sync()
  end
end)
