-- keymapping functions
local km = require('user/modules/keymapping')

----------------------------------------------------------------
-- General
----------------------------------------------------------------

-- Makes 'j' and 'k' moves by one screen line, not text line
km.nnoremap('j','gj')
km.nnoremap('k','gk')

-- Change list
km.nnoremap('g:','g;')

-- Re source current file
km.nnoremap('<leader>gr',':source %<CR>')

-- Autoindent inside braces
km.nnoremap('<leader>=','=i{<CR>')

----------------------------------------------------------------
-- Window movement, resize and splits
----------------------------------------------------------------
--  normal mode
km.nnoremap('<C-h>', '<C-w>h')
km.nnoremap('<C-j>', '<C-w>j')
km.nnoremap('<C-k>', '<C-w>k')
km.nnoremap('<C-l>', '<C-w>l')

--  terminal mode (C-\\ is escaped form of C-\)
km.tnoremap('<C-h>', '<C-\\><C-N><C-w>h')
km.tnoremap('<C-j>', '<C-\\><C-N><C-w>j')
km.tnoremap('<C-k>', '<C-\\><C-N><C-w>k')
km.tnoremap('<C-l>', '<C-\\><C-N><C-w>l')

--  insert mode (C-\\ is escaped form of C-\)
km.inoremap('<C-h>', '<C-\\><C-N><C-w>h')
km.inoremap('<C-j>', '<C-\\><C-N><C-w>j')
km.inoremap('<C-k>', '<C-\\><C-N><C-w>k')
km.inoremap('<C-l>', '<C-\\><C-N><C-w>l')

--  resize if normal mode
km.nnoremap('<A-S-h>', '2<C-w><')
km.nnoremap('<A-S-j>', '2<C-w>-')
km.nnoremap('<A-S-k>', '2<C-w>+')
km.nnoremap('<A-S-l>', '2<C-w>>')

--  splits and tabs
km.nnoremap('<leader>ss', ':sp<CR>')
km.nnoremap('<leader>sv', ':vs<CR>')
km.nnoremap('<leader>st', ':tabnew<CR>')
km.nnoremap('<leader>sc', ':tabclose<CR>')
km.nnoremap(']s', 'gt')
km.nnoremap('[s', 'gT')

--  open tag in splits or new tab
km.nnoremap('<leader>sh', ':topleft  vertical   stag <C-R><C-W><CR>')
km.nnoremap('<leader>sj', ':botright horizontal stag <C-R><C-W><CR>')
km.nnoremap('<leader>sk', ':topleft  horizontal stag <C-R><C-W><CR>')
km.nnoremap('<leader>sl', ':botright vertical stag <C-R><C-W><CR>')
km.nnoremap('<leader>sn', '<C-w><C-]><C-w>T')  -- open tag in new tab
km.nnoremap('<leader>sm', '<C-w>T')  -- move buffer to new tab

----------------------------------------------------------------
-- Save and exit options
----------------------------------------------------------------
km.nnoremap('<leader><C-q>', ':qa<CR>')
km.nnoremap('<leader>w', ':w<CR>')
km.nnoremap('<leader>W', ':wa<CR>')

----------------------------------------------------------------
-- Copy/paste from OS clipboard
----------------------------------------------------------------
km.vnoremap('<leader>gc', '"+y')
km.nnoremap('<leader>gv', '"+p')
km.nnoremap('<leader>gV', '"+P')

----------------------------------------------------------------
-- Terminal mode
----------------------------------------------------------------
-- Esc key
km.tnoremap('<leader><Esc>', '<C-\\><C-n>')

-- To simulate |i_CTRL-R| in terminal-mode:
vim.cmd([[
  tnoremap <expr> <C-R> '<C-\><C-N>"'.nr2char(getchar()).'pi'
]])

-- Open a terminal in a separate horizontal split
vim.cmd([[
  nnoremap <leader>ot :sp \| :te<CR>
]])

----------------------------------------------------------------
-- Search options
----------------------------------------------------------------
km.nnoremap('<CR>', ':nohl<CR>')

-- Make sure to use the last flags when repeating a search
km.nnoremap('&', ':&&<CR>')
km.vnoremap('&', ':&&<CR>')

----------------------------------------------------------------
-- Indent selected text
----------------------------------------------------------------
km.vnoremap('<', '<gv')
km.vnoremap('>', '>gv')

----------------------------------------------------------------
-- Navigate buffers
----------------------------------------------------------------
km.nnoremap('<S-h>', ':bprevious<CR>')
km.nnoremap('<S-l>', ':bnext<CR>')

----------------------------------------------------------------
-- Move text up and down
----------------------------------------------------------------
km.nnoremap('<S-j>', ':m .+1<CR>==')
km.nnoremap('<S-k>', ':m .-2<CR>==')
km.vnoremap('<S-j>', ':move \'>+1<CR>gv-gv')
km.vnoremap('<S-k>', ':move \'<-2<CR>gv-gv')

----------------------------------------------------------------
-- Misc commands
----------------------------------------------------------------
km.nnoremap('<A-h>', 'H')
km.nnoremap('<A-j>', 'J')
km.nnoremap('<A-k>', 'K')
km.nnoremap('<A-l>', 'L')

km.vnoremap('<A-h>', 'H')
km.vnoremap('<A-j>', 'J')
km.vnoremap('<A-k>', 'K')
km.vnoremap('<A-l>', 'L')
