local M = {}

-- TODO: backfill this to template
M.setup = function()
  local signs =
  {
    -- TODO fix char visualization
    -- { name = "DiagnosticSignError", text = "" },
    -- { name = "DiagnosticSignWarn", text = "" },
    -- { name = "DiagnosticSignHint", text = "" },
    -- { name = "DiagnosticSignInfo", text = "" },
    { name = "DiagnosticSignError", text = "E" },
    { name = "DiagnosticSignWarn",  text = "W" },
    { name = "DiagnosticSignHint",  text = "H" },
    { name = "DiagnosticSignInfo",  text = "I" },
  }

  for _, sign in ipairs(signs) do
    vim.fn.sign_define(sign.name, { texthl = sign.name, text = sign.text, numhl = "" })
  end

  local config =
  {
    -- disable virtual text
    virtual_text = false,
    -- show signs
    signs = { active = signs },
    update_in_insert = true,
    underline = true,
    severity_sort = true,
    float =
    {
      focusable = false,
      style = "minimal",
      border = "none",
      source = "always",
      header = "",
      prefix = "",
    }
  }

  vim.diagnostic.config(config)

  vim.lsp.handlers["textDocument/hover"] = vim.lsp.with(vim.lsp.handlers.hover, { border = "none" })
  vim.lsp.handlers["textDocument/signatureHelp"] = vim.lsp.with(vim.lsp.handlers.signature_help, { border = "none" })
end

local function lsp_highlight_document(client)
  -- Set autocommands conditional on server_capabilities
  if client.server_capabilities.document_highlight then
    vim.api.nvim_exec(
      [[
      augroup lsp_document_highlight
        autocmd! * <buffer>
        autocmd CursorHold <buffer> lua vim.lsp.buf.document_highlight()
        autocmd CursorMoved <buffer> lua vim.lsp.buf.clear_references()
      augroup END
      ]],
      false)
  end
end

local  function buf_nnoremap(bufnr, shortcut, command)
  vim.api.nvim_buf_set_keymap(bufnr, "n", shortcut, command, { noremap = true, silent = true })
end

local function lsp_keymaps(bufnr)
  buf_nnoremap(bufnr, "[d",       ":lua vim.lsp.buf.declaration()<CR>")
  buf_nnoremap(bufnr, "]d",       ":lua vim.lsp.buf.definition()<CR>")
  buf_nnoremap(bufnr, "<leader>lk", "<cmd>lua vim.lsp.buf.hover()<CR>")
  buf_nnoremap(bufnr, "<leader>li", "<cmd>lua vim.lsp.buf.implementation()<CR>")
  buf_nnoremap(bufnr, "<leader>ls", "<cmd>lua vim.lsp.buf.signature_help()<CR>")
  buf_nnoremap(bufnr, "<leader>lR", "<cmd>lua vim.lsp.buf.rename()<CR>")
  buf_nnoremap(bufnr, "<leader>lr", "<cmd>lua vim.lsp.buf.references()<CR>")
  buf_nnoremap(bufnr, "<leader>lo", "<cmd>lua vim.lsp.buf.code_action()<CR>")
  buf_nnoremap(bufnr, "<leader>lt", "<cmd>lua vim.lsp.buf.type_definition()<CR>")
  buf_nnoremap(bufnr, "<leader>ln", '<cmd>lua vim.lsp.diagnostic.show_line_diagnostics({ border = "rounded" })<CR>')
  buf_nnoremap(bufnr, "<leader>la", "<cmd>lua vim.diagnostic.open_float()<CR>")
  buf_nnoremap(bufnr, "<leader>ll", "<cmd>lua vim.diagnostic.setloclist()<CR>")
  buf_nnoremap(bufnr, "[w",         '<cmd>lua vim.diagnostic.goto_prev({ border = "rounded" })<CR>')
  buf_nnoremap(bufnr, "]w",         '<cmd>lua vim.diagnostic.goto_next({ border = "rounded" })<CR>')
  buf_nnoremap(bufnr, "<Esc>",      "")
  buf_nnoremap(bufnr, "<Esc>",      "")
  vim.cmd [[ command! Format execute 'lua vim.lsp.buf.formatting()' ]]
end

M.on_attach = function(client, bufnr)
  lsp_keymaps(bufnr)
  lsp_highlight_document(client)
end

-- LSP capabilities
local capabilities = vim.lsp.protocol.make_client_capabilities()

local status_ok, cmp_nvim_lsp = pcall(require, "cmp_nvim_lsp")
if not status_ok then
  vim.notify("cmp_nvim_lsp plugin not found!")
  return
end

M.capabilities = cmp_nvim_lsp.default_capabilities(capabilities)

return M
