" Configuration for lightline

" Load custom color theme
source ~/.config/nvim/colors/lightline/mike_ll.vim

let g:lightline = {}
let g:lightline.colorscheme = 'mike_ll'

let g:lightline.active =
\  {
\     'left' : [ [ 'mode', 'paste' ], [ 'filename' ], [ 'readonly'], [ 'modified' ],
\                [ 'lsp_infos', 'lsp_hints', 'lsp_warnings', 'lsp_errors', 'lsp_ok' ], [ 'lsp_status' ] ],
\     'right': [ [ 'lineinfo' ], [ 'percent' ], [ 'filepath' ] ]
\  }

let g:lightline.mode_map = 
\  {
\   'n' : 'N',
\   'i' : 'I',
\   'R' : 'R',
\   'v' : 'V',
\   'V' : 'VL',
\   "\<C-v>": 'VB',
\   'c' : 'C',
\   's' : 'S',
\   'S' : 'SL',
\   "\<C-s>": 'SB',
\   't': 'T'
\  }

let g:lightline.component_function =
\ {
\   'filepath'    : 'CustomFilePath',
\   'fileencoding': 'CustomFileEncoding',
\ }

"" Add LSP info into lightline
let g:lightline.component_expand =
\ {
\  'readonly'    : 'CustomReadOnly',
\  'lsp_infos'   : 'lightline#lsp#infos',
\  'lsp_hints'   : 'lightline#lsp#hints',
\  'lsp_warnings': 'lightline#lsp#warnings',
\  'lsp_errors'  : 'lightline#lsp#errors',
\  'lsp_ok'      : 'lightline#lsp#ok',
\  'lsp_status'  : 'lightline#lsp#status',
\ }

let g:lightline.component_type =
\ {
\  'lsp_infos'   : 'info',
\  'lsp_hints'   : 'hints',
\  'lsp_warnings': 'warning',
\  'lsp_errors'  : 'error',
\  'lsp_ok'      : 'left',
\ }

let g:lightline.tab =
\ {
\  'active': [ 'tabnum', 'filename' ],
\  'inactive': [ 'tabnum', 'filename' ]
\ }

function! CustomFilePath()
	let l:pathStr = expand('%:p:h')
	let l:winwidth = winwidth(0)
	let l:strlen = strlen(pathStr)
	let l:threshold = 45
	if l:winwidth < l:threshold
		return ''
	elseif l:strlen < (l:winwidth - l:threshold)
    return pathStr
	else
		let l:traillen = l:winwidth - l:threshold - 10
    return strpart(pathStr, 0, 7) . '...' . strpart(pathStr, l:strlen - l:traillen, l:traillen)
	endif
endfunction

function! CustomFileEncoding()
  return winwidth(0) > 85 ? &fileencoding : ''
endfunction

function! CustomReadOnly()
  return &filetype ==# 'help' ? 'H' : &readonly ? 'RO' : ''
endfunction

